"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MDFile = void 0;
const fs = require("fs");
const path = require("path");
const uslug = require("uslug");
const request = require("./utils/request");
const link_1 = require("./link");
class MDFile {
    _absoluteLinks;
    /**
     * This regex should match the following example:
     * * [link](https://example.com)
     * * [link label]: https://exxample.com
     * * <img src="http://example.com/avatar.jpg">
     */
    _absoluteRegex = /(]\(|]:\s*)(https?:\/\/[-a-zA-Z0-9@:%._+~#=]{2,256}\.[a-z]{2,6}\b([-a-zA-Z0-9@:%,*_+.~#?&//=]*))/g;
    _cache;
    _content;
    _ignorePatterns;
    _internalLinks;
    _invalidLinks;
    /**
     * This regex should match the following examples:
     * * [link](#somewhere)
     * * [link] (#somewhere) → invalid link.
     * * [link]( #somewhere)
     */
    _internalRegex = /]\s*\(\s*(#\S*?)\)/g;
    _links;
    _path;
    _relativeLinks;
    _relativePath;
    /**
     * This regex should match the following examples:
     * * [link label]: ./somewhere.md
     * * [link label]: ./somewhere
     * * [link label]: ../somewhere
     * * [link label]:./somewhere.md
     * * [link](./somewhere.md)
     * * [link](./somewhere/)
     */
    _relativeRegex = /]\((\.[^)]*)\)|(]:\s*)(\.[^\s]*)/g;
    _titles;
    _titleRegex = /^#{1,6}\s+(.*)$/gm;
    _normalizedTitles;
    constructor(directory, relativePath, ignorePatterns) {
        this._relativePath = relativePath;
        this._path = path.join(directory, relativePath);
        this._absoluteLinks = new Set();
        this._cache = new Set();
        this._internalLinks = new Set();
        this._relativeLinks = new Set();
        this._titles = new Set();
        this._normalizedTitles = new Set();
        this._ignorePatterns = ignorePatterns;
        this._content = fs.readFileSync(this._path, { encoding: 'utf-8' }); // eslint-disable-line no-sync
        this.getRelativeLinks();
        this.getAbsoluteLinks();
        this.getInternalLinks();
        this.getTitles();
    }
    getRelativeLinks() {
        let val;
        while ((val = this._relativeRegex.exec(this._content)) !== null) {
            const url = val[3] || val[1];
            if (this._cache.has(url)) {
                continue;
            }
            // [link](./somewhere.md) → shift will be 1 because of "]("
            let shift = 2;
            if (!val[1]) {
                /**
                 * [link label]: ./somewhere.md → val[2] will be ": "
                 * [link label]:./somewhere.md → val[2] will be ":"
                 */
                shift = val[2].length;
            }
            this._cache.add(url);
            this._relativeLinks.add(new link_1.Link(url, val.index + shift, this._content));
        }
    }
    getAbsoluteLinks() {
        let val;
        while ((val = this._absoluteRegex.exec(this._content)) !== null) {
            const url = val[2];
            if (this._cache.has(url)) {
                continue;
            }
            /**
             * [link](https://example.com) → val[1] will be "]("
             * [link label]: https://exxample.com → val[1] will be "]: "
             * <img src="http://example.com/avatar.jpg"> → val[1] will be 'src="'
             */
            const shift = val[1].length;
            this._cache.add(url);
            this._absoluteLinks.add(new link_1.Link(url, val.index + shift, this._content));
        }
    }
    getInternalLinks() {
        let val;
        while ((val = this._internalRegex.exec(this._content)) !== null) {
            const url = val[1];
            if (this._cache.has(url)) {
                continue;
            }
            this._cache.add(url);
            this._internalLinks.add(new link_1.Link(url, val.index + 2, this._content));
        }
    }
    getTitles() {
        let val;
        while ((val = this._titleRegex.exec(this._content)) !== null) {
            this._titles.add(val[1]);
            this._normalizedTitles.add(uslug(val[1]));
        }
    }
    ignoreLink(url) {
        return this._ignorePatterns.some((pattern) => {
            return pattern.test(url);
        });
    }
    async validateAbsoluteLinks() {
        const promises = [];
        for (const link of this._absoluteLinks) {
            if (this.ignoreLink(link.link)) {
                link.isValid = true;
                continue;
            }
            const promise = request.get(link.link)
                .then((linkExists) => {
                link.isValid = linkExists;
            });
            promises.push(promise);
        }
        await Promise.all(promises);
    }
    validateRelativeLink(link) {
        if (this.ignoreLink(link.link)) {
            link.isValid = true;
            return;
        }
        const fullPath = path.join(path.dirname(this._path), link.link);
        const [filePath, hash] = fullPath.split('#');
        // Relative links should point to a md file.
        if (!filePath.endsWith('.md')) {
            link.isValid = false;
            return;
        }
        const exists = fs.existsSync(filePath); // eslint-disable-line no-sync
        if (!exists) {
            link.isValid = false;
            return;
        }
        if (!hash) {
            link.isValid = true;
            return;
        }
        /**
         * 1. Read file content
         * 2. Get titles
         * 3. Check if has exists.
         */
        const fileContent = fs.readFileSync(filePath, { encoding: 'utf-8' }); // eslint-disable-line no-sync
        const normalizedTitles = new Set();
        let val;
        while ((val = this._titleRegex.exec(fileContent)) !== null) {
            normalizedTitles.add(uslug(val[1]));
        }
        link.isValid = normalizedTitles.has(hash);
    }
    validateRelativeLinks() {
        this._relativeLinks.forEach((link) => {
            this.validateRelativeLink(link);
        });
    }
    validateInternalLink(link) {
        if (this.ignoreLink(link.link)) {
            link.isValid = true;
            return;
        }
        if (this._normalizedTitles.has(link.link.substr(1))) {
            link.isValid = true;
            return;
        }
        link.isValid = false;
    }
    validateInternalLinks() {
        this._internalLinks.forEach((link) => {
            this.validateInternalLink(link);
        });
    }
    async validateLinks() {
        await this.validateAbsoluteLinks();
        this.validateRelativeLinks();
        this.validateInternalLinks();
    }
    get absoluteLinks() {
        return this._absoluteLinks;
    }
    get internalLinks() {
        return this._internalLinks;
    }
    get path() {
        return this._path;
    }
    get relativeLinks() {
        return this._relativeLinks;
    }
    get relativePath() {
        return this._relativePath;
    }
    get titles() {
        return this._titles;
    }
    get invalidLinks() {
        if (this._invalidLinks) {
            return this._invalidLinks;
        }
        const invalidAbsoluteLinks = [...this._absoluteLinks]
            .filter((link) => {
            return !link.isValid;
        });
        const invalidRelativeLinks = [...this._relativeLinks]
            .filter((link) => {
            return !link.isValid;
        });
        const invalidInternalLinks = [...this._internalLinks]
            .filter((link) => {
            return !link.isValid;
        });
        this._invalidLinks = new Set(invalidAbsoluteLinks.concat(invalidRelativeLinks).concat(invalidInternalLinks));
        return this._invalidLinks;
    }
    get links() {
        if (this._links) {
            return this._links;
        }
        this._links = new Set([...this._absoluteLinks, ...this._relativeLinks, ...this._internalLinks]);
        return this._links;
    }
}
exports.MDFile = MDFile;
//# sourceMappingURL=mdfile.js.map